package main

import (
	"encoding/json"
	"fmt"
	"log"
	"main/pgos_lib"

	"github.com/gofiber/fiber/v2"
)

type PlayerInfoRequest struct {
	PlayerId string `json:"player_id"`
}

func main() {

	app := fiber.New()

	// Route handlers
	app.Get("/", func(c *fiber.Ctx) error {
		return c.SendString("Hello World!")
	})

	// client_hello respond a simple 'Hello' to the invoker, ONLY for client side invoking
	app.Post("/client_hello", func(c *fiber.Ctx) error {
		invoker := c.Get("Invoker")
		if invoker != "client" {
			return c.SendStatus(fiber.StatusUnauthorized)
		}
		name := c.Body()
		log.Println("/client_hello: ", string(c.Body()))
		return c.SendString(fmt.Sprintf("Hello client %s!", name))
	})

	// backend_hello respond a simple 'Hello' to the invoker, ONLY for backend side invoking(by backend http api)
	app.Post("/backend_hello", func(c *fiber.Ctx) error {
		invoker := c.Get("Invoker")
		if invoker != "backend" {
			return c.SendStatus(fiber.StatusUnauthorized)
		}
		name := c.Body()
		log.Println("/backend_hello: ", string(c.Body()))
		return c.SendString(fmt.Sprintf("Hello backend %s!", name))
	})

	// backend_get_player_info is a sample uri to demostrate backend side invoking
	// it invokes backend http api to get a player's info, and respond to the invoker, ONLY for backend side invoking(by backend http api)
	// you can invoke any http api based on your demand, even call other virtual server :-)
	app.Post("/backend_get_player_info", func(c *fiber.Ctx) error {
		invoker := c.Get("Invoker")
		if invoker != "backend" {
			return c.SendStatus(fiber.StatusUnauthorized)
		}
		titleRegionID := c.Get("Titleregionid")
		titleID := c.Get("Titleregionid")

		reqBody := new(PlayerInfoRequest)
		json.Unmarshal(c.Body(), &reqBody)
		req, _ := json.Marshal(reqBody)

		_, body, err := pgos_lib.InvokeBackendApi(titleID, titleRegionID, ServerKey, "/player_svr/get_player_info", req)
		if err != nil {
			return c.SendString(err.Error())
		}

		return c.Send(body)
	})

	// pgos_event handles all event, please config the event handler in the portal in advance
	app.Post("/pgos_event", func(c *fiber.Ctx) error {
		invoker := c.Get("Invoker")
		if invoker != "event" {
			return c.SendStatus(fiber.StatusUnauthorized)
		}

		// Write your own code to handle
		log.Println("/pgos_event: ", string(c.Body()))
		return nil
	})

	// pgos_economy handles the specific requests from PGOS economy service, please config the binding for the economy service on the portal in advance
	app.Post("/pgos_economy", func(c *fiber.Ctx) error {
		invoker := c.Get("Invoker")
		if invoker != "economy" {
			return c.SendStatus(fiber.StatusUnauthorized)
		}

		// Write your own code to handle
		log.Println("/pgos_economy: ", string(c.Body()))
		return nil
	})

	// pgos_task handles the task request, please config the task on the portal in advance
	app.Post("/pgos_task", func(c *fiber.Ctx) error {
		invoker := c.Get("Invoker")
		if invoker != "task" {
			return c.SendStatus(fiber.StatusUnauthorized)
		}

		// Write your own code to handle
		log.Println("/pgos_task: ", string(c.Body()))
		// Just echo the request body to the requester
		return c.SendString(string(c.Body()))

	})

	log.Fatal(app.Listen(":8081"))
}
